// (c) Copyright 2001-2002 iwombat.com all rights reserved
package com.iwombat.foundation.persist;

import java.sql.*;

import java.lang.reflect.*;

import com.iwombat.util.*;

import com.iwombat.foundation.Identifier;

/**
 * Class AbstractBeanToSqlParameterMapper
 * Description: Abstract supporting class to map Bean properties to sql prepared statement parameters
 *
 * <br><br>
 * Data types supported:<br>
 * com.iwombat.Identifier<br>
 * java.lang.String<br>
 * java.lang.Long<br>
 * java.langFloat<br>
 * java.util.Date<br>
 * java.lang.Boolean<br>
 * com.iwombat.ByteArray<br>
 * <br><br>
 * Note that language primitives are not supported because they cannot reflect the concept of 'Null'.
 * <br>
 * @author  $Author: brentp $
 * @version $Revision: 1.1 $
 * 
 * <br>
 * <dl><dt><b>Date:</b><dd>$Date: 2002/11/18 16:04:39 $</dd></dl>
 */

public abstract class AbstractBeanToSqlParameterMapper implements BeanToSqlParameterMapper{
    
    private Object sourceBean;
    private PreparedStatement targetPreparedStatement;
    
    /**
     * Retrieves the target prepared statement object to be used in the mapping operations
     *
     * @return the current set PreparedStatement
     */
    public final PreparedStatement getTargetPreparedStatement() {
        return this.targetPreparedStatement;
    }

    /**
     * Sets the target prepared statement object to be used in the mapping operations
     *
     * @param the target prepared statement
     */
    public final void setTargetPreparedStatement(PreparedStatement targetPreparedStatement) {
        this.targetPreparedStatement=targetPreparedStatement;
    }

    /**
     * Retreives the source Bean to be used in the mapping operations
     *
     * @return the source bean
     */
    public final Object getSourceBean() {
        return this.sourceBean;
    }

    /**
     * Sets the source bean to be used in the mapping operations
     *
     * @param the source bean 
     */
    public final void setSourceBean(Object sourceBean) {
        this.sourceBean=sourceBean;
    }

    /**
     * retrieves the Bean property accessor method
     *
     * @param the bean to retrieve the the method from
     * @param the bean property name to retrieve (e.g. specifying "potato" would return the method named "getPotato")
     */
    protected final Method locateAccessorMethod(
    Object inputBean,
    String beanPropertyName){
        
        Method accessorMethod=null;
        String methodName = "";
        
        try{
            methodName=BeanUtil.createAccessorMethodName(beanPropertyName);
            accessorMethod=inputBean.getClass().getMethod(methodName,new Class[]{});
        }
        catch (NoSuchMethodException e){
            throw new PersistenceException("Could not find the 'get*' method for the '" + beanPropertyName + "' property of the '" + inputBean.getClass().getName() + "'",e);
        }
        return accessorMethod;
    }
    
    /**
     * Writes a prepared statement parameter value based on the specified parameter to bean configuration.
     * Operates against the currently set source bean and target prepared statement
     * 
     * 
     * @param the sql parameter to bean configuration
     */
    public final void writeParameterValue(OrdinalParameterToBeanMapItem paramToBeanConfig) {
        
            Object sourceBean=this.getSourceBean();
            if (null==sourceBean) throw new IllegalStateException("source bean is not defined");
            
            int paramIndex=paramToBeanConfig.getParameterIndex();
            String beanPropertyName=paramToBeanConfig.getBeanPropertyName();
            Method accessorMethod=paramToBeanConfig.getAccessorMethod();
            
            if (accessorMethod==null){
                accessorMethod=locateAccessorMethod(sourceBean,beanPropertyName);
                //cache the method
                paramToBeanConfig.setAccessorMethod(accessorMethod);
            }
            
            Class returnType = accessorMethod.getReturnType();
            Object result=null;

            try{
                result=accessorMethod.invoke(sourceBean,new Object[]{});
            }
            catch (IllegalAccessException e){
                throw new PersistenceException("IllegalAccessException caught: unable to set parameter statement: caught IllegalAccessException while trying to invoke get method '" + accessorMethod.getName() + "' for source bean'" + sourceBean.getClass().getName() + "'", e);
            }
            catch (InvocationTargetException e){
                throw new PersistenceException("InvocationTargetException caught: unable to set parameter statement: caught InvocationTargetException while trying to invoke get method '" + accessorMethod.getName() + "' for source bean '" + sourceBean.getClass().getName() + "'", e);
            }
            
            //Write the bean property get result into the appropriate prepared statement slot
            try{
        
                //Test for String
                if (String.class.isAssignableFrom(returnType)) writeStringParameter((String)result,paramIndex);

                //Test for Identifier
                else if (Identifier.class.isAssignableFrom(returnType)) writeIdentifierParameter((Identifier)result,paramIndex);

                //Test for date
                else if (java.util.Date.class.isAssignableFrom(returnType)) writeDateParameter((java.util.Date)result,paramIndex);

                //Test for Long
                else if (Long.class.isAssignableFrom(returnType)) writeLongParameter((Long)result,paramIndex);

                //Test for Float
                else if (Float.class.isAssignableFrom(returnType)) writeFloatParameter((Float)result,paramIndex);

                //Test for ByteArray
                else if (ByteArray.class.isAssignableFrom(returnType)) writeByteArrayParameter((ByteArray)result, paramIndex);

                //Test for Boolean
                else if (Boolean.class.isAssignableFrom(returnType)) writeBooleanParameter((Boolean)result, paramIndex);
        
                else throw new PersistenceException("Unsupported bean property type: " + returnType.getName() + "; parameter index: " + paramIndex);
            }
            catch (java.sql.SQLException e){
                throw new PersistenceException("Caught SQLException while writing parameter for source Bean '" + sourceBean.getClass().getName() + "'; bean property name '" + beanPropertyName + "'; parameter index: '"+paramIndex+"'");
            }
    }
    
}
