/*--- (c) 2003 iWombat.com All rights reserved ---*/
package com.iwombat.swing;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Point;
import java.beans.PropertyVetoException;

import javax.swing.JDesktopPane;
import javax.swing.JInternalFrame;

/**
 * Scrollable MultiDocumentDesktopPane. This class handles sizing and scroll
 * bar issues with an acompanying parent ScrollPane.
 * 
 * @author  $Author: bob $
 * @version $Revision: 1.2 $
 * 
 * <br>
 * <dl><dt><b>Date:</b><dd>$Date: 2003/10/08 05:16:47 $</dd></dl>
*/

public class MultiDocumentDesktopPane extends JDesktopPane {
    private final static int FRAME_OFFSET = 20;
    private MultiDocumentDesktopManager manager;
    boolean topWindowMoved = false;

    public MultiDocumentDesktopPane() {
        manager = new MultiDocumentDesktopManager(this);
        setDesktopManager(manager);
        setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);
    }
    
    // some convienience methods
    private void syncDesktopSize() {
        if ( (null != getParent()) && this.isVisible() ) {
            manager.syncDesktopSize();
        }
    }

    public void setTopWindowMoved(boolean value) {
        topWindowMoved = value;
    }
    
    public boolean getTopWindowMoved() {
        return topWindowMoved;
    }
    
    public void setBounds(int x, int y, int w, int h) {
        super.setBounds(x,y,w,h);
        syncDesktopSize();
    }

    public Component add(JInternalFrame frame) {
        JInternalFrame[] frames = getAllFrames();
        Point point;
        int width;
        int height;

        Component component = super.add(frame);
        // make sure the desktop can hold it
        syncDesktopSize();
        
        
        // we should really only do this if nobody has moved the windows 
        // TODO: have the DesktopManager reset the moved flag        
        if ( (frames.length > 0) && !this.getTopWindowMoved() ) {
            point = frames[0].getLocation();
            point.x = point.x + FRAME_OFFSET;
            point.y = point.y + FRAME_OFFSET;
        } else {
            point = new Point(0, 0);
        }
        
        frame.setLocation(point.x, point.y);
        
        // lets make sure we can cascade 10 new ones
        if (frame.isResizable()) {
            width = getWidth() - (FRAME_OFFSET * 10);
            height = getHeight() - (FRAME_OFFSET * 10);
            
            // make sure we can fit the window in the desktop
            if (width < frame.getMinimumSize().getWidth()) {
                width = (int)frame.getMinimumSize().getWidth();
            }
            if (height < frame.getMinimumSize().getHeight()) {
                height = (int)frame.getMinimumSize().getHeight();
            }
            frame.setSize(width, height);
        }
        this.moveToFront(frame);
        frame.setVisible(true);
        
        try {
            frame.setSelected(true);
        } catch (PropertyVetoException e) {
            frame.toBack();
        }
        return component;
    }

    public void remove(Component component) {
        super.remove(component);
        syncDesktopSize();
    }

    /**
     * Cascade all internal frames
     */
    public void cascadeFrames() {
        int xpos = 0;
        int ypos = 0;
        JInternalFrame[] frames = getAllFrames();

        manager.setNaturalSize();
        int frameHeight = (getBounds().height - 5) - frames.length * FRAME_OFFSET;
        int frameWidth = (getBounds().width - 5) - frames.length * FRAME_OFFSET;
        
        // all skate in the reverse direction
        for (int i = frames.length-1; i > (-1); i--) {
            frames[i].setSize(frameWidth, frameHeight);
            frames[i].setLocation(xpos, ypos);
            xpos += FRAME_OFFSET;
            ypos += FRAME_OFFSET;
        }
    }

    /**
     * Tile all internal frames
     */
    public void tileFrames() {
        Component frames[] = getAllFrames();
        manager.setNaturalSize();
        int height = getBounds().height / frames.length;
        int ypos = 0;
        for (int i = 0; i < frames.length; i++) {
            frames[i].setSize(getBounds().width, height);
            frames[i].setLocation(0, ypos);
            ypos += height;
        }
    }

    /**
     * Sets all component size properties ( maximum, minimum, preferred)
     * to the given dimension.
     */
    public void setAllSize(Dimension d){
        setMinimumSize(d);
        setMaximumSize(d);
        setPreferredSize(d);
    }

    /**
     * Sets all component size properties ( maximum, minimum, preferred)
     * to the given width and height.
     */
    public void setAllSize(int width, int height){
        this.setAllSize(new Dimension(width, height));
    }


}
